#define F_CPU   16000000UL

#define USART_BAUDRATE 9600
#define BAUD_PRESCALE (((F_CPU/(USART_BAUDRATE*16UL)))-1)

#include <stdbool.h>
#include <stdint.h>
#include <avr/io.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>

const uint16_t pwmtable_8D[32] PROGMEM =
{
	0, 1, 2, 2, 2, 3, 3, 4, 5, 6, 7, 8, 10, 11, 13, 16, 19, 23,
	27, 32, 38, 45, 54, 64, 76, 91, 108, 128, 152, 181, 215, 255
};

static void init( void ) {
	// Setup serial port
	UCSR0B |= (1<<RXEN0)  | (1<<TXEN0);
	UCSR0C |= (1<<UCSZ00) | (1<<UCSZ01);
	UBRR0H  = (BAUD_PRESCALE >> 8);
	UBRR0L  = BAUD_PRESCALE;

	// PD6 is now an output
	DDRD |= (1 << PD6);

	// PD2,3 are inputs for LDR
	DDRD &= ~( (1 << PD2 ) | (1 << PD3 ) );
	PORTD &= ~( 1 << PD2 );
	PORTD &= ~( 1 << PD3 );

	// set none-inverting mode
	TCCR0A |= (1 << COM0A1);

	// set fast PWM Mode
	TCCR0A |= (1 << WGM01) | (1 << WGM00);

	// Enable interrupts:
	sei();
}

char read_serial( void ) {
	char received_byte;
	// wait until a byte is ready to read
	while( ( UCSR0A & ( 1 << RXC0 ) ) == 0 ){}

	received_byte = UDR0;

	return received_byte;
}

void write_serial( char byte ) {
	// wait until the port is ready to be written to
	while( ( UCSR0A & ( 1 << UDRE0 ) ) == 0 ){}

	// write the byte to the serial port
	UDR0 = byte;
}

void print_s( char *text ) {
	char *t;
	for( t = text; *t != '\0'; t++ ) {
		write_serial( *t );
	}
}

void my_delay (uint16_t milliseconds) {
	for(; milliseconds > 0; milliseconds-- )
		_delay_ms( 1 );
}

void pwm_up_down( const uint16_t pwm_table[], int16_t size, uint16_t delay, bool direction ) {
	int16_t tmp;

	// set prescaler to 8 and starts PWM
	TCCR0B |= (1 << CS01);
	// True go up
	if( direction ) {
		for( tmp = 0; tmp < size; tmp++ ) {
			OCR0A = pgm_read_word( & pwm_table[tmp] );
			my_delay( delay );
		}
	// false go down
	} else {
		for( tmp = size-1; tmp >= 0; tmp-- ) {
			OCR0A = pgm_read_word( & pwm_table[tmp] );
			my_delay( delay );
		}
	}
	// stop pwm
	TCCR0B &= ~(1 << CS01);
}

uint8_t readLDR() {
	uint8_t ret = 0x00;
	if( PIND & ( 1 << PD2 ) ) {
		ret += 1;
	}
	if( PIND & ( 1 << PD3 ) ) {
		ret += 2;
	}
	return ret;
}

int main( void ) {
	init();

	print_s( "Start control cycle.\r\n" );

	uint8_t last = 0x00;
	for(;;) {
		if( readLDR() == 1 && last != 1 ) {
			print_s( "LDR 1\r\n" );
			pwm_up_down( pwmtable_8D, 32, 35, true );
			last = 1;
		} else if( readLDR() == 2 && last != 2 ) {
			print_s( "LDR 2\r\n" );
			pwm_up_down( pwmtable_8D, 32, 35, false );
			last = 2;
		}
		my_delay( 10 );
		//	print_s( "Hello Test\r\n" );
	}
	return 0;
}

